"""
**Deal** -- python library for design by contract (DbC) programming.
https://en.wikipedia.org/wiki/Design_by_contract

This library contain 3 main conception from DbC:

* Precondition -- condition that must be true before function is executed.
* Postcondition -- condition that must be true after function executed.
* Invariant -- condition that can be relied upon to be true during execution
  of a program. In this library invariant condition check in 3 cases:
    1. Before class method execution.
    2. After class method execution.
    3. After some class attribute setting.
"""

# main package info
__title__ = 'deal'
__version__ = '2.2.1'
__author__ = 'Gram Orsinium'
__license__ = 'LGPL 3.0'


# version synonym
VERSION = __version__


from .exceptions import *       # noQA
from .aliases import *          # noQA
from .schemes import Scheme     # noQA





class Scheme(object):
    def __init__(self, data, request=None):
        self.data = data

    def is_valid(self):
        raise NotImplementedError  # pragma: no cover


def is_scheme(obj):
    if not hasattr(obj, 'mro'):
        return False
    if Scheme in obj.mro():
        return True
    for parent in obj.mro():
        if parent.__module__.startswith('djburger.'):
            return True
    return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup


setup(
    name='deal',
    version='2.2.1',

    author='orsinium',
    author_email='master_fess@mail.ru',

    description='Programming by contract library.',
    long_description=open('README.rst').read(),
    keywords=(
        'python contracts pre post inv invariant '
        'contracts-programming decorators functional-programming '
        'design-by-contract'),

    packages=['deal'],
    requires=[],

    url='https://github.com/orsinium/deal',
    download_url='https://github.com/orsinium/deal/tarball/master',

    license='GNU Lesser General Public License v3.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Quality Assurance',
    ],
)
